import re
from collections import Mapping

UNDERSCORE_RE = re.compile('[\-_\s]+(.?)')
SPLIT_RE = re.compile('([A-Z][^A-Z]*)')


def _process_keys(obj, fn):
    if isinstance(obj, list):
        return [_process_keys(k, fn) for k in obj]
    elif isinstance(obj, Mapping):
        return {fn(k): _process_keys(v, fn) for k, v in obj.items()}
    else:
        return obj


def separate_words(string, separator='_', split=SPLIT_RE):
    return separator.join([s for s in SPLIT_RE.split(string) if s != ''])


def camelize_keys(obj):
    return _process_keys(obj, camelize)


def camelize(string):
    if string.isnumeric():
        return string
    return ''.join([
        string[0].lower(),
        UNDERSCORE_RE.sub(lambda m: m.group(1).upper(), string)[1:],
    ])


def decamelize_keys(obj):
    return _process_keys(obj, decamelize)


def decamelize(string):
    return separate_words(string).lower()


def depascalize_keys(obj):
    return _process_keys(obj, decamelize)


def depascalize(string):
    return decamelize(string)


def is_camelcase(string):
    return string == camelize(string)


def is_pascalcase(string):
    return string == pascalize(string)


def is_snakecase(string):
    return string == decamelize(string)


def pascalize_keys(obj):
    return _process_keys(obj, pascalize)


def pascalize(string):
    return ''.join([string[0].upper(), camelize(string[1:])])



# -*- coding: utf-8 -*-
# flake8: noqa
# noreorder
"""
Underscore-to-camelCase converter (and vice versa) for strings and dict keys in Python.
"""
__title__ = 'pyhumps'
__version__ = '0.3.0'
__author__ = 'Nick Ficano'
__license__ = 'MIT License'
__copyright__ = 'Copyright 2018 Nick Ficano'

from humps.__main__ import camelize
from humps.__main__ import decamelize
from humps.__main__ import depascalize
from humps.__main__ import pascalize

from humps.__main__ import camelize_keys
from humps.__main__ import decamelize_keys
from humps.__main__ import depascalize_keys
from humps.__main__ import pascalize_keys

from humps.__main__ import is_camelcase
from humps.__main__ import is_pascalcase
from humps.__main__ import is_snakecase



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""This module contains setup instructions for pyhumps."""
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

with open('README.md') as readme_file:
    readme = readme_file.read()

with open('LICENSE') as readme_file:
    license = readme_file.read()

setup(
    name='pyhumps', version='0.3.0', author='Nick Ficano',
    author_email='nficano@gmail.com', packages=['humps'],
    url='https://github.com/nficano/humps', license=license,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console', 'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English', 'Operating System :: MacOS',
        'Operating System :: Microsoft', 'Operating System :: POSIX',
        'Operating System :: Unix',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python', 'Topic :: Utilities', ],
    description=(
        '🐫  Underscore-to-camelCase converter (and vice versa) for strings '
        'and dict keys in Python.'
    ),
    long_description=readme, zip_safe=True,
)
